﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Reflection;

namespace DB.ORM
{

    /// <summary>
    /// 自定义特性帮助类
    /// 通过特性获得相关属性信息
    /// </summary>
    /// <typeparam name="T"></typeparam>
    public class AttributeHelper<T> where T : class, new()
    {

        /// <summary>
        /// 获得某个属性上的自定义特性的
        /// 获得Models特性的Length值
        /// </summary>
        /// <param name="pi"></param>
        /// <returns></returns>
        public static int GetPropertyLength(PropertyInfo pi)
        {
            return GetModelsAttribute(pi).Length;
        }

        /// <summary>
        /// 用于判断是否是 自增长列
        /// </summary>
        /// <param name="pi"></param>
        /// <returns></returns>
        public static bool GetPropertyIsIdentity(PropertyInfo pi)
        {
            return GetModelsAttribute(pi).IsIdentity;
        }

        /// <summary>
        /// 用于判断这个属性是否是主键
        /// </summary>
        /// <param name="pi"></param>
        /// <returns></returns>
        public static bool GetPropertyPrimaryKey(PropertyInfo pi)
        {
            return GetModelsAttribute(pi).IsPrimary;
        }

        /// <summary>
        /// 获得主键属性
        /// 如果没有主键就会抛出错误
        /// </summary>
        /// <param name="t"></param>
        /// <returns></returns>
        public static PropertyInfo GetPrimaryKey(T t)
        {
            PropertyInfo[] propertys = t.GetType().GetProperties();
            foreach (PropertyInfo pi in propertys)
            {
                if (GetModelsAttribute(pi).IsPrimary)
                {
                    return pi;
                }
            }

            throw new InvalidOperationException(t.GetType().ToString() + " is not have primary key ");

        }

        /// <summary>
        /// 获得属性的默认值
        /// </summary>
        /// <param name="pi"></param>
        /// <returns></returns>
        public static object GetPropertyDefault(PropertyInfo pi)
        {
            return GetModelsAttribute(pi).Default;
        }


        /// <summary>
        /// 根据属性的名字获得
        /// 大小写要完全相同,否则会找不到，然后抛出异常 
        /// </summary>
        /// <param name="t"></param>
        /// <param name="propertyName"></param>
        /// <returns></returns>
        public static PropertyInfo GetPropertyInfoByPname(string propertyName)
        {
            T t = new T();

            return t.GetType().GetProperty(propertyName);

            throw new InvalidOperationException(t.GetType().ToString() + " is not have property name(" + propertyName + ")");
        }


        /// <summary>
        /// 获得该属性上面的 特性列表，并且获得映射的特性
        /// 如果改属性没有这个特性的话，会抛出异常
        /// </summary>
        /// <param name="pi"></param>
        /// <returns></returns>
        public static ModelsAttribute GetModelsAttribute(PropertyInfo pi)
        {
            object[] list = pi.GetCustomAttributes(typeof(ModelsAttribute), true);
            ModelsAttribute models = new ModelsAttribute();
            if (list.Length > 0)
            {

                foreach (object item in list)
                {
                    if (item is ModelsAttribute)
                    {
                        models = list[0] as ModelsAttribute; ;
                        return models;
                    }
                }
            }
            throw new Exception(" Not found the attribute in this PropertyInfo ");
        }


    }
}
