﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Reflection;
using System.Data;


using DB.Util;
using DB.ORM;
using System.Data.SqlClient;


namespace DB.ORM
{

    /// <summary>
    /// 实体类反射相关帮助类
    /// </summary>
    /// <typeparam name="T"></typeparam>
    public class ModelReflection<T> where T : class, new()
    {

        /// <summary>
        /// 获得查询语句 
        /// </summary>
        /// <returns></returns>
        public static TsqlCommand Select()
        {
            T t = new T();
            TsqlCommand command = new TsqlCommand();
            StringBuilder sql = new StringBuilder(" SELECT ");
            PropertyInfo[] propertys = t.GetType().GetProperties();
            string tableName = t.GetType().Name;
            StringBuilder fields = new StringBuilder();
            foreach (PropertyInfo pi in propertys)
            {
                ModelsAttribute ma = AttributeHelper<T>.GetModelsAttribute(pi);
                fields.Append(" [" + tableName + "].[" + pi.Name + "],");//将属性名赋值

            }
            sql.Append(fields.ToString().TrimEnd(','));
            sql.AppendFormat(" FROM [dbo].[{0}] ", tableName);

            sql.Append(" WHERE 1=1 ");
            command.TsqlText = sql.ToString();
            return command;
        }


        /// <summary>
        /// 修改 对象
        /// </summary>
        /// <returns></returns>
        public static TsqlCommand Update(T t)
        {
            string tableName = t.GetType().Name;
            TsqlCommand command = new TsqlCommand();

            StringBuilder sql = new StringBuilder(" UPDATE " + tableName + " SET ");
            PropertyInfo[] propertys = t.GetType().GetProperties();

            StringBuilder fields = new StringBuilder();//修改字段的
            string key = string.Empty;//where 条件
            foreach (PropertyInfo pi in propertys)
            {
                string paName = "@" + pi.Name;
                ModelsAttribute ma = AttributeHelper<T>.GetModelsAttribute(pi);
                if (!ma.IsPrimary)//如果不是主键
                {
                    fields.AppendFormat(" [{0}].[{1}] = {2},", tableName, pi.Name, paName);//将属性名赋值
                }
                else//如果是主键
                {
                    key = string.Format(" AND [{0}].[{1}] = {2} ", tableName, pi.Name, paName);
                }
                AddParameter(t, command, pi, ma, paName);
            }
            sql.Append(fields.ToString().TrimEnd(','));

            sql.Append(" WHERE 1=1 ");
            sql.Append(key);//添加where 条件

            command.TsqlText = sql.ToString();
            return command;
        }


        /// <summary>
        /// 添加对象
        /// </summary>
        /// <param name="t"></param>
        /// <returns></returns>
        public static TsqlCommand Insert(T t)
        {
            TsqlCommand command = new TsqlCommand();
            string tableName = t.GetType().Name;
            //获得改模型的所有属性值
            List<PropertyInfo> propertys = t.GetType().GetProperties().ToList();
            //获得自增长列
            //订单修改语句
            StringBuilder sql = new StringBuilder();
            sql.AppendFormat(" INSERT INTO [dbo].[{0}] ( ", tableName);
            StringBuilder sql1 = new StringBuilder();
            StringBuilder sql2 = new StringBuilder();
            //获得参数值
            SqlParameter[] pas = new SqlParameter[propertys.Count - 1];
            int i = 0;
            //循环属性，拼接添加SQL语句
            foreach (PropertyInfo pi in propertys)
            {
                ModelsAttribute ma = AttributeHelper<T>.GetModelsAttribute(pi);
                //如果是自增长列,跳过
                if (ma.IsIdentity)
                {
                    continue;
                }
                string paName = "@" + pi.Name;
                //添加字段
                if (sql1.Length == 0)
                {
                    sql1.Append(pi.Name);
                }
                else
                {
                    sql1.Append("," + pi.Name);
                }
                //添加参数
                if (sql2.Length == 0)
                {
                    sql2.Append(paName);
                }
                else
                {
                    sql2.Append("," + paName);
                }

                AddParameter(t, command, pi, ma, paName);
                i++;
            }
            //累加主键SQL语句
            sql.Append(sql1.ToString() + ") VALUES (");
            sql.Append(sql2.ToString() + "); SELECT SCOPE_IDENTITY();");

            command.TsqlText = sql.ToString();

            return command;
        }




        /// <summary>
        /// 根据主键删除
        /// </summary>
        /// <param name="t">模型</param>
        /// <returns>操作是否成功</returns>
        public static TsqlCommand Delete(object value)
        {
            T t = new T();
            TsqlCommand command = new TsqlCommand();
            string tableName = t.GetType().Name;

            PropertyInfo pi = AttributeHelper<T>.GetPrimaryKey(t);//获得主键名，主键名既数据库字段
            ModelsAttribute ma = AttributeHelper<T>.GetModelsAttribute(pi);
            string paName = "@" + pi.Name;
            //定义删除语句
            StringBuilder sql = new StringBuilder();
            sql.Append(" DELETE FROM " + tableName);
            sql.Append(" WHERE " + pi.Name + "=" + paName + " ");

            command.TsqlText = sql.ToString();

            SqlParameter pa = new SqlParameter(paName, ma.DbType, ma.Length) { Value = value };
            command.Parameters.Add(pa);

            return command;
        }


        /// <summary>
        /// 批量删除
        /// </summary>
        /// <param name="t">模型</param>
        /// <param name="ids">Ids</param>
        /// <returns>操作结果</returns>
        public static TsqlCommand DeleteByIds(string ids)
        {
            T t = new T();
            TsqlCommand command = new TsqlCommand();
            string tableName = t.GetType().Name;

            PropertyInfo pi = AttributeHelper<T>.GetPrimaryKey(t);//获得主键名，主键名既数据库字段
            ModelsAttribute ma = AttributeHelper<T>.GetModelsAttribute(pi);


            //定义删除语句
            string sql = " DELETE FROM " + tableName + " WHERE " + pi.Name + " in (" + ids + ")";
            command.TsqlText = sql;
            return command;

        }

        /// <summary>
        /// 为参数添加sqlpara
        /// </summary>
        /// <param name="t"></param>
        /// <param name="command"></param>
        /// <param name="pi"></param>
        /// <param name="ma"></param>
        /// <param name="paName"></param>
        private static void AddParameter(T t, TsqlCommand command, PropertyInfo pi, ModelsAttribute ma, string paName)
        {
            object value = pi.GetValue(t, null);
            SqlParameter pa = new SqlParameter(paName, ma.DbType, ma.Length) { Value = value };
            command.Parameters.Add(pa);
        }

        #region 增加参数条件

        /// <summary>
        ///  创建参数
        /// </summary>
        /// <param name="command">已经创建好的命令</param>
        /// <param name="para"></param>
        public static void CreateParameter(TsqlCommand command, TsqlParameter para)
        {
            T t = new T();
            Type mType = t.GetType();//
            string tabName = mType.Name;//表名称
            PropertyInfo pi = mType.GetProperty(para.PiName);//相关属性
            ModelsAttribute ma = AttributeHelper<T>.GetModelsAttribute(pi);
            string sql = "";
            string sqlPaName = "@" + para.PiName;
            switch (para.Condition)
            {
                case CompareCondition.BelongTo:
                    sql = string.Format("  AND CHARINDEX('[{1}].[{2}]','{0}') > 0  ", sqlPaName, tabName, para.PiName);

                    break;
                case CompareCondition.Contain:
                    sql = string.Format(" AND  CHARINDEX('{0}','[{1}].[{2}]') > 0  ", sqlPaName, tabName, para.PiName);
                    break;
                case CompareCondition.EqualTo:
                    sql = string.Format(" AND  [{1}].[{2}] = {0} ", sqlPaName, tabName, para.PiName);
                    break;
                case CompareCondition.GreaterThan:
                    sql = string.Format(" AND  [{1}].[{2}] > {0} ", sqlPaName, tabName, para.PiName);
                    break;
                case CompareCondition.GreaterThanEqual:
                    sql = string.Format("  AND [{1}].[{2}] >= {0} ", sqlPaName, tabName, para.PiName);
                    break;
                case CompareCondition.LessThan:
                    sql = string.Format(" AND  [{1}].[{2}] < {0} ", sqlPaName, tabName, para.PiName);
                    break;
                case CompareCondition.LessThanEqual:
                    sql = string.Format(" AND  [{1}].[{2}] <= {0} ", sqlPaName, tabName, para.PiName);
                    break;
            }

            command.TsqlText += sql;
            SqlParameter pa = new SqlParameter(sqlPaName, ma.DbType, ma.Length)
            {
                Value = para.Value
            };
            command.Parameters.Add(pa);
        }


        /// <summary>
        ///  创建参数
        /// </summary>
        /// <param name="command">已经创建好的命令</param>
        /// <param name="paras"></param>
        public static void CreateParameter(TsqlCommand command, IList<TsqlParameter> paras)
        {
            foreach (TsqlParameter para in paras)
            {
                CreateParameter(command, para);
            }

        }

        #endregion

    }
}
