﻿using System;
using System.Collections.Generic;
using System.Text;
using System.Data;
using System.Reflection;

namespace DB.Util
{
    /// <summary>
    /// 实体转换辅助类
    /// </summary>
    class ModelConvertHelper<T> where T : new()
    {

        public static IList<T> ConvertToModel(DataTable dt)
        {
            // 定义集合
            IList<T> ts = new List<T>();

            string tempName = "";

            foreach (DataRow dr in dt.Rows)
            {
                T t = new T();

                // 获得此模型的公共属性
                PropertyInfo[] propertys = t.GetType().GetProperties();

                foreach (PropertyInfo pi in propertys)
                {
                    tempName = pi.Name;

                    // 检查DataTable是否包含此列
                    if (dt.Columns.Contains(tempName))
                    {
                        // 判断此属性是否有Setter
                        if (!pi.CanWrite) continue;

                        object value = dr[tempName];
                        if (value != DBNull.Value)
                            pi.SetValue(t, value, null);
                    }
                }

                ts.Add(t);
            }

            return ts;

        }

        /*
        /// <summary>
        /// 将DataTable转换成为相应List,符合该BPM系统的编码规范
        /// </summary>
        /// <param name="dt"></param>
        /// <returns></returns>
        public static IList<T> ConvertToModel(DataTable dt)
        {
            // 定义集合
            IList<T> ts = new List<T>();

            string tempName = "";

            foreach (DataRow dr in dt.Rows)
            {
                T t = new T();

                // 获得此模型的公共属性
                PropertyInfo[] propertys = t.GetType().GetProperties();

                foreach (PropertyInfo pi in propertys)
                {
                    tempName = pi.Name;//将属性名赋值

                    // 检查DataTable是否包含此列
                    for (int i = 0; i < dt.Columns.Count; i++)
                    {
                        string colPro = dt.Columns[i].ToString();//获得某个列的列名

                        //将列名转换成为属性名，对比获得相关列的值
                        if (ColumnToProperty(colPro) != tempName) continue;

                        // 判断此属性是否有Setter
                        if (!pi.CanWrite) continue;

                        //将列中的值为该属性赋值
                        object value = dr[colPro];
                        if (value != DBNull.Value)
                        {
                            pi.SetValue(t, value, null);
                            break;
                        }
                    }
                }

                ts.Add(t);
            }

            return ts;

        }**/

        /// <summary>
        /// 将列明转换成为属性名，然后用于匹配
        /// </summary>
        /// <param name="column"></param>
        /// <returns></returns>
        private static string ColumnToProperty(string column)
        {
            return ConvertToPascal(ClearUnderLine(ClearProfix(column)));
        }

        //去除前缀，保留前缀后面的_下划线
        private static string ClearProfix(string str)
        {
            int index = str.IndexOf("_");
            return str.Substring(index);
        }

        //获取所有_下划线后面一位单词改成大写
        private static string ClearUnderLine(string str)
        {
            string result = "";
            for (int i = 0; i < str.Length; i++)
            {
                string c = str[i].ToString();
                if (c == "_")
                {
                    c = str[i + 1].ToString().ToUpper();
                    i++;
                }

                result += c;

            }
            return result;
        }

        //帕斯卡命名法
        private static string ConvertToPascal(string str)
        {
            return str.Substring(0, 1).ToUpper() + str.Substring(1);
        }

    }
}
